package com.cesgroup.zw.codetable.web.freemarker.taglib;


import java.io.IOException;
import java.io.Writer;
import java.lang.reflect.Field;
import java.util.Map;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

import com.cesgroup.zw.core.codetable.CodeGroup;
import com.cesgroup.zw.core.codetable.CodeTableCache;
import com.cesgroup.zw.core.codetable.util.StringUtils;

import freemarker.core.Environment;
import freemarker.template.TemplateDirectiveBody;
import freemarker.template.TemplateDirectiveModel;
import freemarker.template.TemplateException;
import freemarker.template.TemplateModel;
/**
 * CODE TABLE 输出自定义标签
 * @author Roger
 *
 */
public class CodeTableWriteTag implements  TemplateDirectiveModel{
	
	public static String TAGNAME = "Write";
	
	private Log logger = LogFactory.getLog(CodeTableWriteTag.class);
	
	// CODETABLE 组的名称
	private final String PARAM_NAME_GROUP_NAME = "groupName";
	// CODETABLE 每一个主键的名称
	private final String PARAM_NAME_KEY = "key";
	// CODETABLE 要显示的字段名称
	private final String PARAM_NAME_DISPLAY_NAME = "displayName";
	
	private final String DEFAULT_DISPLAY_NAME = "display";
	

	
	public void execute(Environment env, Map params, TemplateModel[] loopVars,
			TemplateDirectiveBody body) throws TemplateException, IOException {
		
		// 取得CODETABLE的实例
		CodeTableCache codeTableCache =  CodeTableCache.getInstance();
		// 取得要处理的组的名称
		String groupName = null !=  params.get(PARAM_NAME_GROUP_NAME)?params.get(PARAM_NAME_GROUP_NAME).toString():null;
		// 取得主KEY的名称
		String key = null != params.get(PARAM_NAME_KEY)?params.get(PARAM_NAME_KEY).toString():null;
		
		if(StringUtils.isEmpty(groupName)) {
			logger.warn("业务组名的信息不能为空。");
			return ;
			
		}
		if( StringUtils.isEmpty(key)) {
			logger.warn("业务组名为"+groupName+"的对应的业务信息不能为空。");
			return ;			
		}
		// 取得要显示的字段名称
		String displayName = null != params.get(PARAM_NAME_DISPLAY_NAME)?params.get(PARAM_NAME_DISPLAY_NAME).toString():null;
		if(StringUtils.isEmpty(displayName)) {
			displayName = DEFAULT_DISPLAY_NAME;
		}
		
		try {
			if(codeTableCache.containsGroup(groupName)) {
				
				CodeGroup group = codeTableCache.getGroup(groupName);

				if(group.containsCode(key)) {
					
					Object obj = group.getCode(key);
					Field field = obj.getClass().getDeclaredField(displayName);
					field.setAccessible(true);
					Writer out = env.getOut();  
					String value = (String)field.get(obj);
					if(null != value) {
						out.write(value);
						if(null != body )
							body.render(out);
					}
					
				}
			}
		} catch (Exception e) {
			logger.error("业务组名为"+groupName+"的对应的业务信息取得时发生异常",e);
		}
	}
	
	

}
