package com.cesgroup.zw.spring.boot.autoconfigure;

import javax.annotation.PostConstruct;
import javax.sql.DataSource;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.autoconfigure.AutoConfigureAfter;
import org.springframework.boot.autoconfigure.condition.ConditionalOnClass;
import org.springframework.boot.autoconfigure.condition.ConditionalOnMissingBean;
import org.springframework.boot.autoconfigure.condition.ConditionalOnProperty;
import org.springframework.boot.autoconfigure.condition.ConditionalOnWebApplication;
import org.springframework.boot.autoconfigure.jdbc.DataSourceAutoConfiguration;
import org.springframework.boot.context.properties.EnableConfigurationProperties;
import org.springframework.boot.web.servlet.ServletListenerRegistrationBean;
import org.springframework.context.annotation.Bean;
import org.springframework.context.annotation.Configuration;

import com.cesgroup.zw.codetable.WEBCodeTableCache;
import com.cesgroup.zw.codetable.query.CodeTableQueryDAOImpl;
import com.cesgroup.zw.codetable.web.freemarker.taglib.CodeTableWriteTag;
import com.cesgroup.zw.codetable.web.listener.CodeTableServletContextListener;

@Configuration
@ConditionalOnProperty(name = "codetable.enable", havingValue = "true")
@ConditionalOnClass({ WEBCodeTableCache.class})
@EnableConfigurationProperties(CodeTableProperties.class)
//在指定的配置类初始化后再加载 
@AutoConfigureAfter(DataSourceAutoConfiguration.class)
public class CodeTableAutoConfiguration {
	
	private static final Logger logger = LoggerFactory.getLogger(CodeTableAutoConfiguration.class);

	private CodeTableProperties properties;
	
	
	public CodeTableAutoConfiguration(CodeTableProperties properties) {
		super();
		this.properties = properties;
	}
	
	
	@Bean
	@ConditionalOnMissingBean
	public WEBCodeTableCache webCodeTableCache(DataSource dataSource) {
		CodeTableQueryDAOImpl codeTableQuery =  new CodeTableQueryDAOImpl();
		codeTableQuery.setDataSource(dataSource);
		
		WEBCodeTableCache webCodeTableCache = new WEBCodeTableCache(codeTableQuery);
		
		webCodeTableCache.setXmlCodeFiles(properties.getXmlCodeFiles());
		
		webCodeTableCache.setDbCodeFiles(properties.getDbCodeFiles());
		
		try {
			webCodeTableCache.loadData();
		} catch (Exception e) {
			logger.error(e.getMessage(), e);
		}
		return webCodeTableCache;
	}
	
	@Configuration
	@ConditionalOnProperty(name = "codetable.webEnable", havingValue = "true")
	@ConditionalOnWebApplication
	public class CodeTableWebConfiguration {	
	
		/*@Bean
		public CodeTableWriteTag codeTableWriteTag() {
			return new CodeTableWriteTag();
		}*/

		@Autowired
		private freemarker.template.Configuration configuration;
		
		@PostConstruct
		public void setSharedVariable(){
			configuration.setSharedVariable(CodeTableWriteTag.TAGNAME,new CodeTableWriteTag());//标签名与标签类
		}

		
		@Bean
		public ServletListenerRegistrationBean<CodeTableServletContextListener> serssionListenerBean(WEBCodeTableCache webCodeTableCache){

			ServletListenerRegistrationBean<CodeTableServletContextListener> 

			sessionListener = new ServletListenerRegistrationBean<CodeTableServletContextListener>(new CodeTableServletContextListener(webCodeTableCache));

			return sessionListener;

		}

	
	}
	
	
	
	
	
		

}
