package com.cesgroup.zw.common.parse;

import java.util.HashMap;
import java.util.Map;
import java.util.ServiceLoader;

import com.cesgroup.zw.common.parse.ParameterParse.Type;

/**
 * 解析器的工厂类
 * @author Roger 
 * @version 1.0.0
 */
public class ParameterParseFactory {
	
	
	// 解析器的源
	public static Map<Type,ParameterParse> pool = new HashMap<Type,ParameterParse>();
	
	public static ServiceLoader<ParameterParse> serviceLoader = ServiceLoader.load(ParameterParse.class);
	/**
	 * 根据表达式的内容，生成所需要的解析器
	 * OGNL解析器 解析#arg,#result的表达式
	 * WEB解析器解析#request，#session，#application的表达式
	 * 业务字典解析器解析#code_table的表达式
	 * 环境变量解析器 解析#env的表达式
	 * 默认的解析器解析 ${},%{}，空的表达式
	 * @param expression 要解析的表达式
	 * @return 相应的解析器
	 * @throws Exception
	 */
	public static ParameterParse create(String expression) throws Exception {
		
		Type type = Type.DEFAULT;
		if(expression.startsWith("#")){
			// OGNL解析器 解析#arg,#result的表达式
			if(expression.startsWith("#arg") || expression.startsWith("#result")) {
				type = Type.OGNL;
				
			} // WEB解析器解析#request，#session，#application的表达式
			else if(expression.startsWith("#request") || expression.startsWith("#session") || expression.startsWith("#application"))  {
				type = Type.WEB;
			}  //业务字典解析器解析#code_table的表达式
			else if(expression.startsWith("#code_table"))  {
				type = Type.CODE_TABLE;
			}  // 环境变量解析器 解析#env的表达式
			else if(expression.startsWith("#env"))  {
				type = Type.ENV;
			} //  默认的解析器解析 ${},%{}，空的表达式
			else {
				type = Type.DEFAULT;
			}
		}
		// 从数据源中取得
		ParameterParse parse = pool.get(type);
		if(null == parse) {
			for(ParameterParse parameterParse:serviceLoader) {
				if(parameterParse.getType().equals(type)){
					parse = parameterParse;
					pool.put(type, parse);
				}
			}
		}
		if(null == parse) {
			throw new Exception("对应的解析器没有。"+"type->"+expression);
		}
		
		return parse;
	}

	public Map<Type, ParameterParse> getMap() {
		return ParameterParseFactory.pool;
	}

	public void setMap(Map<Type, ParameterParse> map) {
		ParameterParseFactory.pool = map;
	}
	

}
