package com.cesgroup.zw.common.parse.util;

import java.io.File;
import java.io.InputStream;

import org.apache.commons.digester.Digester;
import org.apache.commons.digester.xmlrules.DigesterLoader;
import org.xml.sax.InputSource;

/**
 * 配置文件映射成BEAN的工具
 * @author Roger 
 * @version 1.0.0
 *
 */
@SuppressWarnings("unchecked")
public class DigesterUtil {
	
	/**
	 * 根据配置文件和规则文件将一个配置文件转换成一个具体的类
	 * @param <T>
	 * @param clz
	 * @param inputStream 配置文件的流
	 * @param ruleName 规则文件名
	 * @return 映射成BEAN
	 * @throws Exception
	 */
	public static <T> T parseXML(Class<T> clz,File file,String ruleName) throws Exception {
		InputStream inputRuleStream = clz.getClassLoader().getResourceAsStream(clz.getPackage().getName().replaceAll("[.]", "/")+"/"+ruleName+".xml");
		InputSource source = new InputSource(inputRuleStream);
		Digester digester = DigesterLoader.createDigester(source); 
		try {
			return (T)digester.parse(file);
		} catch(Exception e) {			
			throw e;
		}
	}	
	
	/**
	 * 根据配置文件和规则文件将一个配置文件转换成一个具体的类
	 * @param <T>
	 * @param clz
	 * @param inputStream 配置文件的流
	 * @param ruleName 规则文件名
	 * @return 映射成BEAN
	 * @throws Exception
	 */
	public static <T> T parseXML(Class<T> clz,InputStream inputStream,String ruleName) throws Exception {
		InputStream inputRuleStream = clz.getClassLoader().getResourceAsStream(clz.getPackage().getName().replaceAll("[.]", "/")+"/"+ruleName+".xml");
		InputSource source = new InputSource(inputRuleStream);
		Digester digester = DigesterLoader.createDigester(source); 
		try {
			return (T)digester.parse(inputStream);
		} catch(Exception e) {			
			throw e;
		}
	}
	
	/**
	 * 根据配置文件和规则文件将一个配置文件转换成一个具体的类
	 * @param <T>
	 * @param clz
	 * @param inputStream 配置文件
	 * @param ruleName 规则文件名
	 * @return 映射成BEAN
	 * @throws Exception
	 */	
	public static <T> T  parseXML(Class<T> clz,String fileName,String ruleName) throws Exception {
		
		InputStream inputStream = clz.getClassLoader().getResourceAsStream(fileName); 
		InputStream inputRuleStream = clz.getClassLoader().getResourceAsStream(clz.getPackage().getName().replaceAll("[.]", "/")+"/"+ruleName+".xml");
		InputSource source = new InputSource(inputRuleStream);
		Digester digester = DigesterLoader.createDigester(source); 
		try {
			return (T)digester.parse(inputStream);
		} catch(Exception e) {			
			throw e;
		}
		
	}
	
}
