package com.cesgroup.zw.core.codetable;

import java.lang.reflect.Field;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Collections;
import java.util.HashMap;
import java.util.LinkedHashMap;
import java.util.List;
import java.util.Map;
import java.util.Set;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

import com.cesgroup.zw.core.codetable.util.StringUtils;

@SuppressWarnings({"unchecked","rawtypes"})
public class CodeGroup<T> {
	
	private Log log = LogFactory.getLog(CodeGroup.class);

    private Object cacheKey = null;

    private Map<String, T> codeMap = null;

    private String keyField = null;

    private Class<T> codeClass = null;

    public CodeGroup(Map<String, T> codeMap, Object cacheKey, Class<T> codeClass, String keyField) {

        super();
        this.codeMap = codeMap;
        if (codeMap == null) {
            codeMap = new LinkedHashMap<String, T>();
        }
        this.cacheKey = cacheKey;
        this.codeClass = codeClass;
        this.keyField = keyField;
    }

    
    public Class<T> getCodeClass() {
    	return codeClass;
    }
    
    public String getKeyField() {
    	return keyField;
    }
    
    public String[] getCodeKeys() {

        Set<String> keySet = codeMap.keySet();
        String[] codeKeys = (String[]) keySet.toArray(new String[0]);

        return codeKeys;
    }
    
    
    public T[] getCodes() {
    	if(null == codeMap || codeMap.values() == null ) {
    		return (T[])new  Object[]{};
    	}
        return (T[])(codeMap.values().toArray());
    }

    public List<T> getCodes(String valueName,String value) {
    	
        List<T> beanList = new ArrayList<T>();
        try {
	        for (T code : getCodes()) {
	        	Field field = null;
	            field =  code.getClass().getDeclaredField(valueName);
	            field.setAccessible(true);
	            Object ob  = field.get(code);
				if (null != ob && String.valueOf(ob).equals(value)) {
				    beanList.add(code);
				}
	        }
        } catch (Exception e) {
        	log.error("数据转换异常", e);
		} 
		return null;
    }
    
    public Map<String, T> getCodeMap(String valueName,String value) {
    	
    	Map<String, T> beanMap = new HashMap<String, T>();
        try {
	        for (Map.Entry<String, T> code : codeMap.entrySet()) {
	        	Field field = null;
	            field =  code.getValue().getClass().getDeclaredField(valueName);
	            field.setAccessible(true);
	            Object ob  = field.get(code.getValue());
				if (null != ob && String.valueOf(ob).equals(value)) {
					beanMap.put(code.getKey(), code.getValue());
				}
	        }
        } catch (Exception e) {
        	log.error("数据转换异常", e);
		} 
		return beanMap;
    }
  
    public List<T> getCodes(String valueName) {

        List<T> beanList = new ArrayList<T>();
        Field field = null;

        try {

            for (T code : getCodes()) {
                if (field == null) {
                    field =  code.getClass().getDeclaredField(valueName);
                    field.setAccessible(true);
                }
                if (null != field.get(code)) {
                    beanList.add(code);
                }
            }

            return beanList;

        } catch (Exception e) {
        	log.error("数据转换异常", e);
        }
        return beanList;
    }


    public Object[] getCodeValues(String valueName) {

        List<Object> valueList = new ArrayList<Object>();
        Field field = null;

        try {

            for (Object code : getCodes()) {
                if (field == null) {
                    field = code.getClass().getDeclaredField(valueName);
                    field.setAccessible(true);
                }
                valueList.add(field.get(code));
            }

            return valueList.toArray();

        } catch (Exception e) {
        	log.error("数据转换异常", e);
        }
        return null;
    }


    public int getCodeCount() {

        return codeMap.size();
    }

    public T getCode(String key) {

        if (!containsCode(key)) {
           
        }
        return codeMap.get(key);
    }


    public T getCodeByIndexOf(int index) {

        if (getCodeCount() - 1 < index) {
           
        }
        return getCodes()[index];
    }


    public boolean containsCode(String key) {

        return codeMap.containsKey(key);
    }

    public CodeGroup<T> getASC(String valueName) {

        return getSortByValueName(valueName, Order.ASC, true);
    }

    public CodeGroup<T> getASCByNum(String valueName) {

        return getSortByValueName(valueName, Order.ASC, false);
    }
    
    public CodeGroup<T> getASCSortByValueName(String valueName) {

        return getSortByValueName(valueName, Order.ASC, true);
    }

    public CodeGroup<T> getASCSortByValueNameNum(String valueName) {

        return getSortByValueName(valueName, Order.ASC, false);
    }

    public CodeGroup<T> getDESC(String valueName) {

        return getSortByValueName(valueName, Order.DESC, true);
    }

    public CodeGroup<T> getDESCByNum(String valueName) {

        return getSortByValueName(valueName, Order.DESC, false);
    }   
    
    public CodeGroup<T> getDESCSortByValueName(String valueName) {

        return getSortByValueName(valueName, Order.DESC, true);
    }

    public CodeGroup<T> getDESCSortByValueNameNum(String valueName) {

        return getSortByValueName(valueName, Order.DESC, false);
    }

    private CodeGroup<T> getSortByValueName(String fieldName, Order order, boolean type) {
    	if(codeMap == null || codeMap.values() == null) {
    		return this;
    	}
        List<T> cacheList = (List<T>) Arrays.asList(codeMap.values().toArray());

        String sortField = StringUtils.isEmpty(fieldName) ? keyField : fieldName;

        if (type) {

            Collections.sort(cacheList, new FieldObjectComparator<T>(codeClass, sortField, order));

        } else {

            Collections.sort(cacheList, new FieldNumberComparator<T>(codeClass, sortField, order));
        }

        Map<String, T> beanMap = new LinkedHashMap<String, T>();
        try {
            Field field = codeClass.getDeclaredField(keyField);
            field.setAccessible(true);

            for (T code : cacheList) {
                beanMap.put(String.valueOf(field.get(code)), code);
            }

        } catch (Exception e) {
        	log.error("数据转换异常", e);
        }

        return new CodeGroup(beanMap, cacheKey, codeClass, keyField);
    }


	@Override
	public String toString() {
		return "CodeGroup [cacheKey=" + cacheKey + ", codeMap=" + codeMap + "]";
	}
    

}
