package com.cesgroup.zw.core.codetable;

import java.io.InputStream;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.HashSet;
import java.util.List;
import java.util.Map;
import java.util.Set;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

import com.cesgroup.zw.common.parse.query.ParseQuery;
import com.cesgroup.zw.core.codetable.xml.CodeTableXml;
import com.cesgroup.zw.core.codetable.xml.XMLGroup;

@SuppressWarnings({"unchecked","rawtypes"})
public class CodeTableCache implements Context  {
	
	
	private Log log = LogFactory.getLog(CodeTableCache.class);

	private static CodeTableCache cache = new CodeTableCache();
	
	private Map<String, CodeGroup> cacheMap = new HashMap<String, CodeGroup>();

	private Map<String, Group> dbCacheMap = new HashMap<String, Group>();
	
	private Map<String, String> codeTypeMap = new HashMap<String, String>();

	private Map<String, String> keyFieldMap = new HashMap<String, String>();

	private Map<String, String> xPathMap = new HashMap<String, String>();

	private Map<String, CodeTableXml> r2XmlMap = new HashMap<String, CodeTableXml>();

	private List<String> cacheKeyList = new ArrayList<String>();
	
	private Set<Object> dataSourceTypes = new HashSet<Object>();
	
	private CodeTableCache() {
		
	}
	
	public static  CodeTableCache getInstance(){
		return cache;
	}

	public void loadData() throws Exception {
		
		if(null != xmlCodeInputStreams) {
			for(InputStream xmlCodeInputStream:xmlCodeInputStreams) {
				CodeTableXml xml = new CodeTableXml(xmlCodeInputStream);
				List<XMLGroup> xmlGroupList = xml.getXMLGroupList();
				
				for (XMLGroup xmlGroup : xmlGroupList) { 
					
			        xPathMap.put(xmlGroup.getGroupName(), xmlGroup.getXpath());
			        r2XmlMap.put(xmlGroup.getGroupName(), xml);
		
			        Map<String, ? extends Object> codeMap = xml.copyToBeanMap(xmlGroup.getXpath(), 
			        		xmlGroup.getClazz(), "name", xmlGroup.getPrimaryField());
			        CodeGroup codeGroup = new CodeGroup(codeMap, xmlGroup.getGroupName(),
			        		xmlGroup.getClazz(), xmlGroup.getPrimaryField());
			        cacheMap.put(xmlGroup.getGroupName(), codeGroup);
			        
			        if(null == xmlGroup.getClazz()){
			        	log.error("业务字典的组【"+xmlGroup.getGroupName()+"】的类【"+xmlGroup.getClazz()+"】不存在，请确认是否编译。");
			        	throw new Exception("业务字典的组【"+xmlGroup.getGroupName()+"】的类【"+xmlGroup.getClazz()+"】不存在，请确认是否编译。");
			        }
			        
			        this.setDefinition(xmlGroup.getGroupName(), xmlGroup.getClazz(), xmlGroup.getPrimaryField());
					
				}
			}
		}
		
		if(null != dbCodeInputStreams) {
			for(InputStream dbCodeInputStream:dbCodeInputStreams) {

				DBCodeTableMaker maker = new DBCodeTableMaker(query);
				DBCodeTable dbCodeTable = maker.makeDBCodeTable(dbCodeInputStream);
				List<Group> groupList = dbCodeTable.getGroupList();
				for (Group group : groupList) {
					String cacheKey = group.getName();
					String keyField = group.getKeyMapping().getField();
					
					dbCacheMap.put(cacheKey, group);
					
					if(!dataSourceTypes.isEmpty()) {
						
						
						for(Object ob:dataSourceTypes) {
							
							
							Map codeMap = maker.createDbCodeGroup(group,
									group.getClazz(), cacheKey, keyField);
							CodeGroup codeGroup = new CodeGroup(codeMap, cacheKey, group.getClazz(),
									keyField);
							cacheMap.put(reBuildGroupKey(cacheKey), codeGroup);							
							
						}
						
					} else {
					
						// 多数据源考虑
						Map codeMap = maker.createDbCodeGroup(group,
								group.getClazz(), cacheKey, keyField);
						CodeGroup codeGroup = new CodeGroup(codeMap, cacheKey, group.getClazz(),
								keyField);
						cacheMap.put(cacheKey, codeGroup);
					}
					
			        if(null == group.getClazz()){
			        	log.error("业务字典的组【"+cacheKey+"】的类【"+group.getClazz()+"】不存在，请确认是否编译。");
			        	throw new Exception("业务字典的组【"+cacheKey+"】的类【"+group.getClazz()+"】不存在，请确认是否编译。");
			        }
					setDefinition(cacheKey, group.getClazz(), keyField);
				}
			}
		}
		
	}	
	
	public void initMethod() throws Exception {
		
		//loadData();
		
		/*
		if(null != xmlCodeFiles) {
			for(String xmlFile:xmlCodeFiles) {
				CodeTableXml xml = new CodeTableXml(xmlFile);
				List<XMLGroup> xmlGroupList = xml.getXMLGroupList();
				
				for (XMLGroup xmlGroup : xmlGroupList) { 
					
			        xPathMap.put(xmlGroup.getGroupName(), xmlGroup.getXpath());
			        r2XmlMap.put(xmlGroup.getGroupName(), xml);
		
			        Map<String, ? extends Object> codeMap = xml.copyToBeanMap(xmlGroup.getXpath(), 
			        		xmlGroup.getClazz(), "name", xmlGroup.getPrimaryField());
			        CodeGroup codeGroup = new CodeGroup(codeMap, xmlGroup.getGroupName(),
			        		xmlGroup.getClazz(), xmlGroup.getPrimaryField());
			        cacheMap.put(xmlGroup.getGroupName(), codeGroup);
			        
			        this.setDefinition(xmlGroup.getGroupName(), xmlGroup.getClazz(), xmlGroup.getPrimaryField());
					
				}
			}
		}
		if(null != dbCodeFiles) {
			for(String dbFile:dbCodeFiles) {
				DBCodeTableMaker maker = new DBCodeTableMaker(query);
				DBCodeTable dbCodeTable = maker.makeDBCodeTable(dbFile);
				List<Group> groupList = dbCodeTable.getGroupList();
				for (Group group : groupList) {
					String cacheKey = group.getName();
					String keyField = group.getKeyMapping().getField();
					Map codeMap = maker.createDbCodeGroup(group,
							group.getClazz(), cacheKey, keyField);
					CodeGroup codeGroup = new CodeGroup(codeMap, cacheKey, group.getClazz(),
							keyField);
					cacheMap.put(cacheKey, codeGroup);
					dbCacheMap.put(cacheKey, group);
					setDefinition(cacheKey, group.getClazz(), keyField);
				}
			}
		}*/
	}
	
	public void reload(String groupKey) throws Exception {
		
		if(cacheKeyList.contains(groupKey)) {
			
            Class clazz = getClass(groupKey);
            String keyField = keyFieldMap.get(groupKey);

            CodeTableXml xml = r2XmlMap.get(groupKey);
            if (xml != null) {
                String xpath = xPathMap.get(groupKey);

                Map<String, ? extends Object> codeMap = xml.copyToBeanMap(xpath, clazz,"name", keyField);
                CodeGroup codeGroup = new CodeGroup(codeMap, groupKey, clazz, keyField);
                cacheMap.put(groupKey, codeGroup);
            } else {
            	DBCodeTableMaker maker = new DBCodeTableMaker(query);
            	Group group = dbCacheMap.get(groupKey);
            	
            	
            	Map codeMap = maker.createDbCodeGroup(group,
						group.getClazz(), groupKey, keyField);
				CodeGroup codeGroup = new CodeGroup(codeMap, groupKey, group.getClazz(),
						keyField);
				cacheMap.put(reBuildGroupKey(groupKey), codeGroup);
				
            }
		}
	}
	

	public void setDataSourceTypes(Set<Object> dataSourceTypes) {
		this.dataSourceTypes = dataSourceTypes;
	}
	
	public Set<Object>  getDataSourceTypes() {
		return this.dataSourceTypes;
	}
	
	private String reBuildGroupKey(String groupKey) {

		return groupKey;
	}
	
	public boolean containsGroup(String cacheKey) throws Exception {
		return getGroup(cacheKey) != null;
	}

	private <T extends Object> void setDefinition(String cacheKey,
			Class<T> clazz, String keyField) {
		if (!cacheKeyList.contains(cacheKey)) {
			cacheKeyList.add(cacheKey);
		}
		codeTypeMap.put(cacheKey, clazz.getCanonicalName());
		keyFieldMap.put(cacheKey, keyField);
	}

	public synchronized CodeGroup getGroup(String cacheKey) throws Exception {
		if (!cacheMap.containsKey(reBuildGroupKey(cacheKey))) {
			/*
			Class clazz = getClass(cacheKey);
			String keyField = keyFieldMap.get(cacheKey);
			if (clazz == null || null == keyField) {
				return null;
			}
			CodeGroup codeGroup = createCodeGroup(cacheKey, clazz, keyField);
			cacheMap.put(reBuildGroupKey(cacheKey), codeGroup);*/
			this.reload(cacheKey);
		}
		return cacheMap.get(reBuildGroupKey(cacheKey));
	}

	private Class getClass(String cacheKey) {
		Class clazz = null;
		try {
			String className = codeTypeMap.get(cacheKey);
			if (null == className) {
				return null;
			}
			clazz = Class.forName(className);
		} catch (ClassNotFoundException e) {

		}
		return clazz;
	}

	public synchronized <T extends Object> CodeGroup<T> getGroup(
			String cacheKey, Class<T> clazz) throws Exception {
		return getGroup(cacheKey);
	}

	private <T extends Object> CodeGroup<T> createCodeGroup(String key,
			Class<T> clazz, String keyField) throws Exception {
		DBCodeTableMaker maker = new DBCodeTableMaker(query);
		Group group = dbCacheMap.get(key);
		Map<String, T> codeMap = maker.createDbCodeGroup(group,clazz, key, keyField);
		return new CodeGroup(codeMap, key, clazz, keyField);

	}

	public String[] getGroupNames() {
		return cacheKeyList.toArray(new String[cacheKeyList.size()]);
	}


	

	private ParseQuery query;

	// XML数据流
	private InputStream[] dbCodeInputStreams ;
	
	// DB数据流
	private InputStream[] xmlCodeInputStreams ;
	
	
	private List<String> dbCodeFiles = new ArrayList<String>();
	
	private List<String> histDBCodeFiles = new ArrayList<String>();
	
	private List<String> xmlCodeFiles = new ArrayList<String>();
	private List<String> histXMLCodeFiles = new ArrayList<String>();
	
	public void setDbCodeFiles(List<String> dbCodeFiles) {
		
		this.dbCodeFiles = dbCodeFiles;
		
		histDBCodeFiles.addAll(this.dbCodeFiles);
	}

	public void setXmlCodeFiles(List<String> xmlCodeFiles) {
		this.xmlCodeFiles = xmlCodeFiles;
		this.histXMLCodeFiles.addAll(this.xmlCodeFiles);
	}


	public void setQuery(ParseQuery query) {
		this.query = query;
	}

	public void setDbCodeInputStreams(InputStream[] dbCodeInputStreams) {
		this.dbCodeInputStreams = dbCodeInputStreams;
	}

	public void setXmlCodeInputStreams(InputStream[] xmlCodeInputStreams) {
		this.xmlCodeInputStreams = xmlCodeInputStreams;
	}



}
