package com.cesgroup.zw.core.codetable;

import java.io.File;
import java.io.InputStream;
import java.lang.reflect.Field;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.LinkedHashMap;
import java.util.List;
import java.util.Map;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

import com.cesgroup.zw.common.parse.query.ParseQuery;
import com.cesgroup.zw.common.parse.util.DigesterUtil;


@SuppressWarnings({"unchecked","unused"})
public class DBCodeTableMaker {
	
	private Log log = LogFactory.getLog(DBCodeTableMaker.class);
	
	private ParseQuery query;
	
	public DBCodeTableMaker(ParseQuery query) {
		this.query = query;
	}

	public DBCodeTable makeDBCodeTable(String dbCodeDef) throws Exception {
		
		try{
			
			File file = new File(getClass().getClassLoader().getResource(dbCodeDef).getFile());
		
			return DigesterUtil.parseXML(DBCodeTable.class, file, "db-code-table-define-Rule");
			
		} catch(Exception e){
			log.error("",e);
			throw new Exception(e);
		}
	}

	public DBCodeTable makeDBCodeTable(InputStream inputStream) throws Exception {
		try{
			return DigesterUtil.parseXML(DBCodeTable.class, inputStream, "db-code-table-define-Rule");
		}catch(Exception e){
			log.error("",e);
			throw new Exception(e);
		}
	}
	
	public <T extends Object> Map<String, T> createDbCodeGroup(
			String dbCodeFile, Class<T> clazz, String key, String keyField)
			throws Exception {

		DBCodeTable codeTable = makeDBCodeTable(dbCodeFile);

		for (Group group : codeTable.getGroupList()) {

			if (null == group.getName() || "".equals(group.getName())
					|| null == group.getSql()) {
				continue;
			}
			return createDbCodeGroup(group, group.getClazz(), key, keyField);
		}

		return null;

	}

	public <T extends Object> Map<String, T> createDbCodeGroup(Group group,
			Class<T> clazz, String key, String keyField) throws Exception {

		List<Map<String, Object>> beanMapList = new ArrayList<Map<String, Object>>();
		List<ValueMapping> valueMappingList = group.getValueMappingList();

		
		String groupName = group.getName();
	
		String sql = group.getSql();
		if ((null != groupName && key.equals(groupName)) && null != sql) {
		
			Map<String, Object> paramMap = new HashMap<String, Object>();

			beanMapList = query.queryForList(sql);

			List<Map<String, Object>> tempList = new ArrayList<Map<String, Object>>();
			for (Map<String, Object> beanMap : beanMapList) {
				tempList.add(replaceBeanMapKey(beanMap, valueMappingList));
			}

			List<T> codeList = new ArrayList<T>();
			for (Map<String, Object> beanMap : tempList) {

				T codeClass = clazz.newInstance();
				
				Field[] fields = clazz.getDeclaredFields();
				
				for(Field field:fields) {
					if(beanMap.containsKey(field.getName())) {
						field.setAccessible(true);
						field.set(codeClass, beanMap.get(field.getName()));
					}
				}
				codeList.add(codeClass);
			}

			return beanListToMap(codeList, clazz, keyField);

		}
		return null;
	}

	
	private Map<String, Object> replaceBeanMapKey(Map<String, Object> beanMap,
			List<ValueMapping> valueMappingList) {

		Map<String, Object> tempBeanMap = new HashMap<String, Object>();

		for (ValueMapping valueMapping : valueMappingList) {
			String valueName = valueMapping.getName();
			String valueField = valueMapping.getField();
			Object value = beanMap.get(valueName);
			tempBeanMap.put(valueField, value);
		}
		return tempBeanMap;
	}


	private <T extends Object> Map<String, T> beanListToMap(List<T> beanList,
			Class<T> clazz, String keyField) {

		Map<String, T> map = new LinkedHashMap<String, T>();
		try {
			Field field = clazz.getDeclaredField(keyField);
			field.setAccessible(true);
			for (T bean : beanList) {
				
				map.put(field.get(bean).toString(), bean);
			}
		} catch (Exception e) {
			e.printStackTrace();
		}
		return map;

	}
}
