package com.cesgroup.zw.core.codetable.util;



import java.lang.reflect.Array;
import java.lang.reflect.Field;
import java.util.ArrayList;
import java.util.Collection;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

import com.cesgroup.zw.core.codetable.CodeGroup;
import com.cesgroup.zw.core.codetable.CodeTableCache;


public class CodeTableUtil {
	
	private static Log logger = LogFactory.getLog(CodeTableUtil.class);
	/**
	 * 根据主键从业务字典取得相应的对象
	 * @param <T>
	 * @param clz
	 * @param groupName
	 * @param primaryId
	 * @return
	 */
	public static <T> T getCodeByKey(Class<T> clz,String groupName,String primaryId) {
		
		CodeTableCache codeTableCache = CodeTableCache.getInstance();
		
		try {
			return codeTableCache.getGroup(groupName,clz).getCode(primaryId);
		} catch (Exception e) {
			logger.error( e.getMessage(),e);
		}
		return null;
		
	}

	/**
	 * 根据主键从业务字典取得相应的对象
	 * @param <T>
	 * @param clz
	 * @param groupName
	 * @param primaryId
	 * @return
	 */
	public static Object getCodeByKey(String groupName,String primaryId) {
		
		CodeTableCache codeTableCache = CodeTableCache.getInstance();
		
		try {
			return codeTableCache.getGroup(groupName).getCode(primaryId);
		} catch (Exception e) {
			logger.error( e.getMessage(),e);
		}
		return null;
		
	}
	
	public static Object[] getCodeValues(String groupName,String valueName) {
		
		CodeTableCache codeTableCache = CodeTableCache.getInstance();
		
		try {
			
			CodeGroup group = codeTableCache.getGroup(groupName);
			
			return group.getCodeValues(valueName);
			
		} catch (Exception e) {
			
		}
		
		return null;
		
	}
	
	/**
	 * 根据组名称从业务字典取得相应的对象
	 * @param <T>
	 * @param clz
	 * @param groupName
	 * @return
	 */
	public static <T> List<T> getGroupByName(Class<T> clz,String groupName) {
		
		List<T> dataList = new ArrayList<T>();
		
		CodeTableCache codeTableCache = CodeTableCache.getInstance();

		try {
			CodeGroup<T> group = codeTableCache.getGroup(groupName,clz);

			for(T t:group.getCodes()) {
				dataList.add(t);
			}
		} catch (Exception e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		return dataList;
		
	}
	
	

	
	/**
	 * 根据组名称和相应的字段值从业务字典取得相应的对象
	 * @param <T>
	 * @param clz 
	 * @param groupName 业务字典的组名
	 * @param valueName 相应的字段
	 * @param value 相应的字段的值
	 * @param isSystemData 	支付类型【是否系统主句状态 系统数据时：不可删除】（true:系统数据 false非系统数据）
	 * @return
	 * @throws Exception
	 */
	public static <T> Collection<T> getGroupByNameSelective(Class<T> clz,String groupName,String valueName,String value,boolean isSystemData) throws Exception {
		
		CodeTableCache codeTableCache = CodeTableCache.getInstance();
		// 根据组名取得相应的组信息
		CodeGroup<T> group = codeTableCache.getGroup(groupName,clz);
		
		Map<String,T> result = new HashMap<String,T>();
		
		// 根据过滤条件取得相应的数据
		Map<String, T> dataMap = group.getCodeMap(valueName, value);
		if(null != dataMap) {
			result.putAll(dataMap);
		}
		if(isSystemData) {
			// 根据过滤条件取得相应的数据
			 dataMap = group.getCodeMap("isSystemData", "0");
			 if(null != dataMap) {
				result.putAll(dataMap);
			 }	
		}
		return result.values();

	}
	


	@SuppressWarnings("unchecked")
	public static <T>  T[] getCodes( T[] codes,String valueName,String value) {
		Class<T> clz = null;
		List<T> list = new ArrayList<T>();
	    try {
	        for (T ob:codes) {
	        	clz = (Class<T>) ob.getClass();
	        	Field field = null;
	            field =  ob.getClass().getDeclaredField(valueName);
	            field.setAccessible(true);
	            Object val  = field.get(ob);
				if (null != val && String.valueOf(val).equals(value)) {
					list.add(ob);
				}
	        }
	    } catch (Exception e) {
	    	logger.error(e.getMessage(), e);
		} 
	    if(!list.isEmpty()) {
	    	T[] tt = (T[]) Array.newInstance(clz, list.size());  
	    	return list.toArray(tt);
	    } else {
	    	return null;
	    }		
	}
	public static <T>  List<T> getCodeByNameAndFieldName(Class<T> clz,String groupName,String fieldName,Object fieldValue) throws Exception{
		List<T> list=CodeTableUtil.getGroupByName(clz, groupName);
		return ConvertUtil.groupByFieldName(fieldName,list).get(fieldValue);
	}
	public static Object getFieldValueByKey(String groupName, String enterpriseId,
			String fieldName) throws Exception {
		Object obj=CodeTableUtil.getCodeByKey(groupName, enterpriseId);
		if(obj==null)
			return null;
		return FieldUtil.getValue(obj, fieldName);
	}
	
}
