package com.cesgroup.zw.core.codetable.util;


import java.util.ArrayList;
import java.util.HashMap;
import java.util.LinkedHashMap;
import java.util.List;
import java.util.Map;

public class ConvertUtil {
	
	/**
	 * 根据fieldName的值进行分组
	 * @param <T>
	 * @param clz
	 * @param fieldName 分组的关键字
	 * @param dataList 要分组的集合
	 * @return
	 * @throws Exception
	 */
	public static <T> Map<Object,List<T>> groupByFieldName(Class<T> clz,String fieldName,List<T> dataList) throws Exception {
		
		Map<Object,List<T>> map = new HashMap<Object,List<T>>();
		
		for(T bean:dataList) {
			// 取出到分组的值
			Object value = FieldUtil.getValue(bean, fieldName);
			List<T> list = map.get(value);
			if(null == list) {
				list = new ArrayList<T>();
				map.put(value, list);
			}
			list.add(bean);
		}
		
		return map;
	}
	
	/**
	 * 根据fieldName的值进行分组
	 * @param <T>
	 * @param clz
	 * @param fieldName 分组的关键字
	 * @param dataList 要分组的集合
	 * @return
	 * @throws Exception
	 */
	public static <T> Map<Object,T> groupByField(Class<T> clz,String fieldName,List<T> dataList) throws Exception {
		
		Map<Object,T> map = new HashMap<Object,T>();
		
		for(T bean:dataList) {
			// 取出到分组的值
			Object value = FieldUtil.getValue(bean, fieldName);
			
			map.put(value, bean);
		}
		return map;
	}
	
	/**
	 * 根据fieldName的值进行分组
	 * @param <T>
	 * @param fieldName 分组的关键字
	 * @param dataList 要分组的集合
	 * @return
	 * @throws Exception
	 */
	public static <T> Map<Object,T> groupByField(String fieldName,List<T> dataList) throws Exception {
		
		Map<Object,T> map = new HashMap<Object,T>();
		
		for(T bean:dataList) {
			// 取出到分组的值
			Object value = FieldUtil.getValue(bean, fieldName);
			
			map.put(value, bean);
		}
		return map;
	}
	
	/**
	 * 根据fieldName的值进行分组,按key排序
	 * @param <T>
	 * @param fieldName 分组的关键字
	 * @param dataList 要分组的集合
	 * @return
	 * @throws Exception
	 */
	public static <T> Map<Object,T> groupByFieldSort(String fieldName,List<T> dataList) throws Exception {
		
		Map<Object,T> map = new LinkedHashMap<Object,T>();
		
		for(T bean:dataList) {
			// 取出到分组的值
			Object value = FieldUtil.getValue(bean, fieldName);
			
			map.put(value, bean);
		}
		return map;
	}
	
	/**
	 * map 转换为List
	 * @param <T>
	 * @param clz
	 * @param fieldName 分组的关键字
	 * @param dataList 要分组的集合
	 * @return
	 */
	public static <T> List<T> mapToList(Map<Object,T> map){
		
		List<T> list = new ArrayList<T>();
		
		for(Map.Entry<Object,T> entry :map.entrySet()) {
			// 取出到分组的值
			T value = entry.getValue();
			list.add(value);
		}
		return list;
	}
	
	/**
	 * map 转换为List
	 * @param <T>
	 * @param clz
	 * @param fieldName 分组的关键字
	 * @param dataList 要分组的集合
	 * @return
	 */
	public static <T> List<T> mapListToList(Map<String,List<T>> map){
		
		List<T> list = new ArrayList<T>();
		
		for(Map.Entry<String,List<T>> entry :map.entrySet()) {
			// 取出到分组的值
			List<T> value = entry.getValue();
			list.addAll(value);
		}
		
		return list;
	}
	
	public static <T> Map<Object, List<T>> groupByFieldName(String fieldName,
			List<T> dataList) throws Exception {
		Map<Object, List<T>> map = new HashMap<Object, List<T>>();
		for (T bean : dataList) {
			// 取出到分组的值
			String[] fieldNames = fieldName.split("\\.");
			Object value = bean;
			for (String fn : fieldNames) {
				value = FieldUtil.getValue(value, fn);
			}
			List<T> list = map.get(value);
			if (null == list) {
				list = new ArrayList<T>();
				map.put(value, list);
			}
			list.add(bean);
		}
		return map;
	}
	public static <T> Map<Object, List<T>> groupByFieldName(
			List<T> dataList,String ... fieldName) throws Exception {
		Map<Object, List<T>> map = new HashMap<Object, List<T>>();
		for (T bean : dataList) {
			// 取出到分组的值
			String key="";
			for(String fn:fieldName){
				String[] fieldNames = fn.split("\\.");
				Object value = bean;
				for (String fName : fieldNames) {
					value = FieldUtil.getValue(value, fName);
				}
				key+=value;
			}
			List<T> list = map.get(key);
			if (null == list) {
				list = new ArrayList<T>();
				map.put(key, list);
			}
			list.add(bean);
		}
		return map;
	}
	/**
	 * @author LinXiaodong
	 * @createDate 2013-12-17
	 * @param str
	 * @param splitFlg
	 * @Desc 将String 转换成 String[]
	 * @return
	 */
	public static String[] strConvertToArray(String str,String splitFlg){
		String[] array = null;
		if(str != null && str !=""){
			if(str.indexOf(splitFlg) > 0){
				array = str.split(",");
			}else{
				array = new String[]{str};
			}
		}
		return array;
	}
}
