package com.cesgroup.zw.core.codetable.util;


import java.beans.BeanInfo;
import java.beans.Introspector;
import java.beans.PropertyDescriptor;
import java.lang.reflect.Field;
import java.lang.reflect.Method;
import java.util.HashMap;
import java.util.Map;
/**
 * 类变量共通类
 * @author Roger
 * @version 1.0 
 */
public class FieldUtil {
	/**
	 * 取得一个实例的变量的值
	 * @param ob 类实例
	 * @param fieldNm 类实例的变量名
	 * @return 指定变量的值
	 * @throws Exception 
	 */
	public static Object getValue(Object ob,String fieldNm) throws Exception  {
		// 类实例为空，返回NULL
		if(null == ob){
			return null;
		}
		Field field = null;
		try {
			// 取得申名的变量名
			field = getDeclaredField(ob,fieldNm);
			field.setAccessible(true);
			return field.get(ob);
		} catch (Exception e) {
			throw e;
		}
	}
	
	public static Field getDeclaredField(Object object, String fieldName){  
        Field field = null ;  
          
        Class<?> clazz = object.getClass() ;  
          
        for(; clazz != Object.class ; clazz = clazz.getSuperclass()) {  
            try {  
                field = clazz.getDeclaredField(fieldName) ;  
                return field ;  
            } catch (Exception e) {  
   
            }   
        }  
      
        return null;  
    } 
	/**
	 * 
	 * @param o
	 * @param fieldName
	 * @return
	 * @throws Exception
	 */
	public static Object getFieldValueByName( Object o,String fieldName) throws Exception {  
       try {    
           String firstLetter = fieldName.substring(0, 1).toUpperCase();  
           
           String getter = "get" + firstLetter + fieldName.substring(1);
           
           Method method = o.getClass().getMethod(getter, new Class[] {}); 
           
           Object value = method.invoke(o, new Object[] {});    
           
           return value;    
       } catch (Exception e) { 
    	   
           throw e; 
       }    
	} 
	
	/**
	 * 对指定的实例对象设定其变量的值
	 * @param ob 类实例
	 * @param fieldNm 类实例的变量名
	 * @param value 
	 * @throws Exception 
	 */
	public static void setValue(Object ob,String fieldNm,Object value) throws Exception  {
		// 类实例为空，返回NULL
		if(null == ob){
			return ;
		}
		Field field = null;
		try {
			// 取得申名的变量名
			field = ob.getClass().getDeclaredField(fieldNm);
			field.setAccessible(true);
			field.set(ob,value);
		} catch (Exception e) {
			throw e;
		}
	}
	
	public static Map<String,String> getStringFileds(Object ob) {
		Map<String,String> map = new HashMap<String,String>();
		try{
			BeanInfo beanInfo = Introspector.getBeanInfo(ob.getClass());
			PropertyDescriptor[] pds = beanInfo.getPropertyDescriptors();
			
			for(PropertyDescriptor pd : pds)
			{
				 Method method = pd.getReadMethod();
				 if(method.getReturnType().equals(String.class)) {
					 map.put(pd.getName(), (String)method.invoke(ob)) ;
				 }
	
			}
		}catch(Exception e) {
			
		}
		return map;
	}
	
	/**
	 * 取得所有BEAN属性
	 * @param ob
	 * @return
	 */
	public  static <T> Map<String,Object> getFileds(Class<T> clz,Object ob) {
		Map<String,Object> map = new HashMap<String,Object>();
		try{
			BeanInfo beanInfo = Introspector.getBeanInfo(ob.getClass());
			
			PropertyDescriptor[] pds = beanInfo.getPropertyDescriptors();
			
			for(PropertyDescriptor pd : pds)
			{
				if(!StringUtils.equals(pd.getName(), "class")) {
					Method method = pd.getReadMethod();
					map.put(pd.getName(), method.invoke(ob)) ;
				}
			}
			
		}catch(Exception e) {
			
		}
		return map;
	} 
	

	/**
	 * 取得所有BEAN属性
	 * @param ob
	 * @return
	 */
	public  static <T> Map<String,Method> getFileds(Class<T> clz) {
		Map<String,Method> map = new HashMap<String,Method>();
		try{
			BeanInfo beanInfo = Introspector.getBeanInfo(clz);
			
			PropertyDescriptor[] pds = beanInfo.getPropertyDescriptors();
			
			for(PropertyDescriptor pd : pds)
			{
				if(!StringUtils.equals(pd.getName(), "class")) {
					map.put(pd.getName(),pd.getWriteMethod());
				}
				 
			}
			
		}catch(Exception e) {
			
		}
		return map;
	} 
	
	/**
	 * 取得一个实例的父类的变量的值
	 * @param ob 类实例
	 * @param fieldNm 类实例的变量名
	 * @return 指定变量的值
	 * @throws Exception 
	 */
	public static Object getSuperValue(Object ob,String fieldNm) throws Exception  {
		// 类实例为空，返回NULL
		if(null == ob){
			return null;
		}
		Field field = null;
		try {
			// 取得申名的变量名
			field = ob.getClass().getSuperclass().getDeclaredField(fieldNm);
			field.setAccessible(true);
			return field.get(ob);
		} catch (Exception e) {
			throw e;
		}
	}
}
