package com.cesgroup.zw.core.codetable.xml;


import java.util.ArrayList;
import java.util.LinkedHashMap;
import java.util.List;
import java.util.Map;
import java.util.regex.Pattern;

import com.cesgroup.zw.core.codetable.util.StringUtils;

@SuppressWarnings("unchecked")
public class XmlNode {
	

	private static final long serialVersionUID = -2224946250718469933L;
	

	private static final Pattern VAL_PATTERN = Pattern.compile("[^@]+\\(.+\\)");
	

	private static final Pattern ATT_PATTERN = Pattern.compile(".+@.+\\(.+\\)");



	protected String name;
	

	protected Object value;
	

	protected Map<String, String> attributes;

	protected List<XmlNode> children;
	

	protected XmlNode parent;
	

	public XmlNode(String name) {
		this.name = name;
		this.attributes = new LinkedHashMap<String, String>();
		this.children = new ArrayList<XmlNode>();
	}
	

	public String getName() {
		return name;
	}


	public void setName(String name) {
		this.name = name;
	}


	public Object getValue() {
		return value;
	}


	public <T extends Object> T getValue(Class<T> clazz) {
		return (T)value;
	}


	public void setValue(Object value) {
		this.value = value;
	}
	

	public Map<String, String> getAttributes() {
		return attributes;
	}


	public void setAttributes(Map<String, String> attributes) {
		this.attributes = attributes;
	}
	

	public void addAttribute(String name, String value) {
		this.attributes.put(name, value);
	}


	public List<XmlNode> getChildren() {
		return children;
	}
	

	public List<XmlNode> getChildren(String qname) {
		List<XmlNode> childList = new ArrayList<XmlNode>();
		for (XmlNode node : this.children) {
			if (node.getName().equals(qname)) {
				childList.add(node);
			}
		}
		return childList;
	}
	

	public List<XmlNode> getChildrenByValue(String qname, Object value) {
		List<XmlNode> childList = new ArrayList<XmlNode>();
		for (XmlNode node : this.children) {
			if (node.getName().equals(qname) &&
					StringUtils.equals((String)node.getValue(), (String)value)) {
				childList.add(node);
			}
		}
		return childList;
	}
	

	public List<XmlNode> getChildrenByAttribute(String qname, String name, String value) {
		List<XmlNode> childList = new ArrayList<XmlNode>();
		for (XmlNode node : this.children) {
			if (node.getName().equals(qname) &&
					StringUtils.equals((String)node.getAttributes().get(name), value)) {
				childList.add(node);
			}
		}
		return childList;
	}
	

	public void addChild(XmlNode child) {
		this.children.add(child);
	}
	

	public XmlNode getParent() {
		return parent;
	}
	

	public void setParent(XmlNode parent) {
		this.parent = parent;
	}


	public List<XmlNode> getNodeListByXPath(String xpath) {
		List<XmlNode> nodeList = new ArrayList<XmlNode>();
		if (xpath == null) {
			return nodeList;
		}
		List<XmlNode> xmlNodeList =  this.getChildren();
		
		//String[] paths = xpath.split("[/]");
		for(XmlNode xmlNode:xmlNodeList) {
			
			if("group".equals(xmlNode.getName())) {
				List<XmlNode> codeXmlNodes = xmlNode.getChildren();
				
				String tempPath = xmlNode.getName() + "@name("+xmlNode.getAttributes().get("name")+")";
				if(xpath.startsWith(tempPath)) {
					for(XmlNode codeNode:codeXmlNodes) {
						if("code".equals(codeNode.getName())) {
							
							nodeList.add(codeNode);
						}						
					}					
				}
			}
		}
		return nodeList;
	}

}
